#include "dsi_misc.hpp"
#include "dsi_reg_info.cpp"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "tracer.hpp"

RegInfo DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[TOTAL_CNT_DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED];

void DSI_cl::enable() {
  DSICONTROLLER->DEVICE_READY_REG = SET_DSI_CONTROLLER_MODEL_DEVICE_READY_REG_DEVICE_READY(DSICONTROLLER->DEVICE_READY_REG, 1);
}

void DSI_cl::disable() {
  DSICONTROLLER->DEVICE_READY_REG = SET_DSI_CONTROLLER_MODEL_DEVICE_READY_REG_DEVICE_READY(DSICONTROLLER->DEVICE_READY_REG, 0);
}

void DSI_cl::enable_DFE() {
  DSICONTROLLER->RST_ENB_DFE_REG = SET_DSI_CONTROLLER_MODEL_RST_ENB_DFE_REG_ENABLE(DSICONTROLLER->RST_ENB_DFE_REG, 1);
}

void DSI_cl::disable_DFE() {
  DSICONTROLLER->RST_ENB_DFE_REG = SET_DSI_CONTROLLER_MODEL_RST_ENB_DFE_REG_ENABLE(DSICONTROLLER->RST_ENB_DFE_REG, 0);
}

int DSI_cl::get_irq_bit() {
  return DSI_LCD_IRQ_BIT;
}

void DSI_cl::maskIrq(/*DSI_irq_type*/ int dsi_irq) {
#ifdef INTEL_TARGET
  DSICONTROLLER->INTR_EN_REG = DSICONTROLLER->INTR_EN_REG | (dsi_irq & DSI_irq_all);
#else
  DSICONTROLLER->INTR_EN_REG |= dsi_irq & DSI_irq_all;
#endif
}

void DSI_cl::unmaskIrq(/*DSI_irq_type*/ int dsi_irq) {
#ifdef INTEL_TARGET
  DSICONTROLLER->INTR_EN_REG = DSICONTROLLER->INTR_EN_REG & ~(dsi_irq & DSI_irq_all);
#else
  DSICONTROLLER->INTR_EN_REG &= ~(dsi_irq & DSI_irq_all);
#endif
}

bool DSI_cl::isIrqMasked(/*DSI_irq_type*/ int dsi_irq) {
  return DSICONTROLLER->INTR_EN_REG & dsi_irq;
}

void DSI_cl::clearIrq(/*DSI_irq_type*/ int dsi_irq) {
  DSICONTROLLER->INTR_STAT_REG = dsi_irq & DSI_irq_all;
}

bool DSI_cl::isIrqActive(/*DSI_irq_type*/ int dsi_irq) {
  return DSICONTROLLER->INTR_STAT_REG & dsi_irq;
}

unsigned int DSI_cl::getIrqActive() {
  return DSICONTROLLER->INTR_STAT_REG;
}

void DSI_cl::set_config(/*DSI_config_type*/int *DSI_config_p) {
  DSI_config_type *DSI_config;
  DSI_config = (DSI_config_type *)DSI_config_p;
  int buf_reg;
  
  buf_reg = DSICONTROLLER->DSI_FUNC_PRG_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_LANES_PRG_REG(buf_reg, DSI_config->data_lines);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_VIDEO_MODE(buf_reg, DSI_config->ch_video_mode);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_COMMAND_MODE(buf_reg, DSI_config->ch_cmd_mode);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_FORMAT_VIDEO_MODE(buf_reg, DSI_config->video_format);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_WIDTH_COMMAND_MODE(buf_reg, DSI_config->cmd_data_width);
  DSICONTROLLER->DSI_FUNC_PRG_REG = buf_reg;
  
  DSICONTROLLER->HS_TX_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_HS_TX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->HS_TX_TIMEOUT_REG, DSI_config->hs_tx_timeout);
  DSICONTROLLER->LP_RX_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_LP_RX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->LP_RX_TIMEOUT_REG, DSI_config->lp_rx_timeout);
  DSICONTROLLER->TURN_AROUND_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_TURN_AROUND_TIMEOUT_REG_TIMEOUT_REGISTER(DSICONTROLLER->TURN_AROUND_TIMEOUT_REG, DSI_config->turn_around_timeout);
  DSICONTROLLER->DEVICE_RESET_REG = SET_DSI_CONTROLLER_MODEL_DEVICE_RESET_REG_DEVICE_RESET_TIMER(DSICONTROLLER->DEVICE_RESET_REG, DSI_config->dev_reset_timer);
  DSICONTROLLER->INIT_COUNT_REG = SET_DSI_CONTROLLER_MODEL_INIT_COUNT_REG_MASTER_INITIALISATION(DSICONTROLLER->INIT_COUNT_REG, DSI_config->init_counter);
  
  DSICONTROLLER->VIDEO_MODE_FORMAT_REG = SET_DSI_CONTROLLER_MODEL_VIDEO_MODE_FORMAT_REG_VIDEO_MODE_FORMAT(DSICONTROLLER->VIDEO_MODE_FORMAT_REG, DSI_config->video_mode);
  DSICONTROLLER->CLK_EOT_REG = DSI_config->clk_eot_reg;
  buf_reg = DSICONTROLLER->DPI_RESOLUTION_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_HOR_ADDRESS_COUNT(buf_reg, DSI_config->DPI_resolution_h);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_VERT_ADDRESS_COUNT(buf_reg, DSI_config->DPI_resolution_v);
  DSICONTROLLER->DPI_RESOLUTION_REG = buf_reg;
  
  DSICONTROLLER->HSYNC_COUNT_REG = DSI_config->HSYNC_count;
  if (DSI_config->HSYNC_bpc >= HSYNC_bpc_min)
    DSICONTROLLER->HORIZ_BACK_PORCH_COUNT_REG = DSI_config->HSYNC_bpc;
  else
    DSICONTROLLER->HORIZ_BACK_PORCH_COUNT_REG = HSYNC_bpc_min;
  if (DSI_config->HSYNC_fpc >= HSYNC_fpc_min)
    DSICONTROLLER->HORIZ_FRONT_PORCH_COUNT_REG = DSI_config->HSYNC_fpc;
  else
    DSICONTROLLER->HORIZ_FRONT_PORCH_COUNT_REG = HSYNC_fpc_min;
  DSICONTROLLER->HORIZ_ACTIVE_AREA_COUNT_REG = DSI_config->HSYNC_aac;
  DSICONTROLLER->VSYNC_COUNT_REG = DSI_config->VSYNC_count;
  if (DSI_config->VSYNC_bpc >= VSYNC_bpc_min)
    DSICONTROLLER->VERT_BACK_PORCH_COUNT_REG = DSI_config->VSYNC_bpc;
  else
    DSICONTROLLER->VERT_BACK_PORCH_COUNT_REG = VSYNC_bpc_min;
  if (DSI_config->VSYNC_fpc >= VSYNC_fpc_min)
    DSICONTROLLER->VERT_FRONT_PORCH_COUNT_REG = DSI_config->VSYNC_fpc;
  else
    DSICONTROLLER->VERT_FRONT_PORCH_COUNT_REG = VSYNC_fpc_min;
  
  DSICONTROLLER->POLARITY_REG = DSI_config->polarity;
  DSICONTROLLER->LP_BYTECLK_REG = SET_DSI_CONTROLLER_MODEL_LP_BYTECLK_REG_BYTECLK(DSICONTROLLER->LP_BYTECLK_REG, DSI_config->LP_byteclk);
  
  buf_reg = DSICONTROLLER->DPHY_PARAM_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_PREP(buf_reg, DSI_config->dphy_hs_prep);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_ZERO(buf_reg, DSI_config->dphy_hs_zero);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_TRAIL(buf_reg, DSI_config->dphy_hs_trail);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_EXIT(buf_reg, DSI_config->dphy_hs_exit);
  DSICONTROLLER->DPHY_PARAM_REG = buf_reg;
  buf_reg = DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_PREP(buf_reg, DSI_config->clk_hs_prep);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_ZERO(buf_reg, DSI_config->clk_hs_zero);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_TRAIL(buf_reg, DSI_config->clk_hs_trail);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_EXIT(buf_reg, DSI_config->clk_hs_exit);
  DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG = buf_reg;
  DSICONTROLLER->HIGH_LOW_SWITCH_COUNT_REG = SET_DSI_CONTROLLER_MODEL_HIGH_LOW_SWITCH_COUNT_REG_DATA_LANE_SWITCH_COUNT(DSICONTROLLER->HIGH_LOW_SWITCH_COUNT_REG, DSI_config->lane_high_low_switch_count);
  buf_reg = DSICONTROLLER->CLK_LANE_SWT_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_HS_TO_LP_SWITCHING_COUNTER(buf_reg, DSI_config->hs_to_lp_switch_count);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_LP_TO_HS_SWITCHING_COUNTER(buf_reg, DSI_config->lp_to_hs_switch_count);
  DSICONTROLLER->CLK_LANE_SWT_REG = buf_reg;
  DSICONTROLLER->AUTO_ERR_REC_REG = DSI_config->err_auto_recovery;
}

void DSI_cl::set_reduce_config(/*DSI_config_type*/int *DSI_config_p) {
  DSI_config_type *DSI_config;
  DSI_config = (DSI_config_type *)DSI_config_p;
  int buf_reg;
  
  buf_reg = DSICONTROLLER->DSI_FUNC_PRG_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_LANES_PRG_REG(buf_reg, DSI_config->data_lines);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_VIDEO_MODE(buf_reg, DSI_config->ch_video_mode);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_COMMAND_MODE(buf_reg, DSI_config->ch_cmd_mode);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_FORMAT_VIDEO_MODE(buf_reg, DSI_config->video_format);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_WIDTH_COMMAND_MODE(buf_reg, DSI_config->cmd_data_width);
  DSICONTROLLER->DSI_FUNC_PRG_REG = buf_reg;
  
  DSICONTROLLER->HS_TX_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_HS_TX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->HS_TX_TIMEOUT_REG, DSI_config->hs_tx_timeout);
  DSICONTROLLER->LP_RX_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_LP_RX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->LP_RX_TIMEOUT_REG, DSI_config->lp_rx_timeout);
  DSICONTROLLER->TURN_AROUND_TIMEOUT_REG = SET_DSI_CONTROLLER_MODEL_TURN_AROUND_TIMEOUT_REG_TIMEOUT_REGISTER(DSICONTROLLER->TURN_AROUND_TIMEOUT_REG, DSI_config->turn_around_timeout);
  DSICONTROLLER->DEVICE_RESET_REG = SET_DSI_CONTROLLER_MODEL_DEVICE_RESET_REG_DEVICE_RESET_TIMER(DSICONTROLLER->DEVICE_RESET_REG, DSI_config->dev_reset_timer);
  DSICONTROLLER->INIT_COUNT_REG = SET_DSI_CONTROLLER_MODEL_INIT_COUNT_REG_MASTER_INITIALISATION(DSICONTROLLER->INIT_COUNT_REG, DSI_config->init_counter);
  
  DSICONTROLLER->VIDEO_MODE_FORMAT_REG = SET_DSI_CONTROLLER_MODEL_VIDEO_MODE_FORMAT_REG_VIDEO_MODE_FORMAT(DSICONTROLLER->VIDEO_MODE_FORMAT_REG, DSI_config->video_mode);
  DSICONTROLLER->CLK_EOT_REG = DSI_config->clk_eot_reg;
  
  DSICONTROLLER->POLARITY_REG = DSI_config->polarity;
  DSICONTROLLER->AUTO_ERR_REC_REG = DSI_config->err_auto_recovery;
}

void DSI_cl::get_config(/*DSI_config_type*/int *DSI_config_p) {
  DSI_config_type *DSI_config;
  DSI_config = (DSI_config_type *)DSI_config_p;
  
  DSI_config->data_lines = (DSI_DataLines)GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_LANES_PRG_REG(DSICONTROLLER->DSI_FUNC_PRG_REG);
  DSI_config->ch_video_mode = (DSI_virt_ch)GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_VIDEO_MODE(DSICONTROLLER->DSI_FUNC_PRG_REG);
  DSI_config->ch_cmd_mode = (DSI_virt_ch)GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_CH_COMMAND_MODE(DSICONTROLLER->DSI_FUNC_PRG_REG);
  DSI_config->video_format = (DSI_video_format)GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_FORMAT_VIDEO_MODE(DSICONTROLLER->DSI_FUNC_PRG_REG);
  DSI_config->cmd_data_width = (DSI_cmd_data_width)GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_WIDTH_COMMAND_MODE(DSICONTROLLER->DSI_FUNC_PRG_REG);
  
  DSI_config->hs_tx_timeout = GET_DSI_CONTROLLER_MODEL_HS_TX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->HS_TX_TIMEOUT_REG);
  DSI_config->lp_rx_timeout = GET_DSI_CONTROLLER_MODEL_LP_RX_TIMEOUT_REG_TIMEOUT_COUNTER(DSICONTROLLER->LP_RX_TIMEOUT_REG);
  DSI_config->turn_around_timeout = GET_DSI_CONTROLLER_MODEL_TURN_AROUND_TIMEOUT_REG_TIMEOUT_REGISTER(DSICONTROLLER->TURN_AROUND_TIMEOUT_REG);
  DSI_config->dev_reset_timer = GET_DSI_CONTROLLER_MODEL_DEVICE_RESET_REG_DEVICE_RESET_TIMER(DSICONTROLLER->DEVICE_RESET_REG);
  DSI_config->init_counter = GET_DSI_CONTROLLER_MODEL_INIT_COUNT_REG_MASTER_INITIALISATION(DSICONTROLLER->INIT_COUNT_REG);
  
  DSI_config->video_mode = (DSI_video_mode)GET_DSI_CONTROLLER_MODEL_VIDEO_MODE_FORMAT_REG_VIDEO_MODE_FORMAT(DSICONTROLLER->VIDEO_MODE_FORMAT_REG);
  DSI_config->clk_eot_reg = (DSI_clk_eot)(int)DSICONTROLLER->CLK_EOT_REG;
  DSI_config->DPI_resolution_h = GET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_HOR_ADDRESS_COUNT(DSICONTROLLER->DPI_RESOLUTION_REG);
  DSI_config->DPI_resolution_v = GET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_VERT_ADDRESS_COUNT(DSICONTROLLER->DPI_RESOLUTION_REG);
  
  DSI_config->HSYNC_count = DSICONTROLLER->HSYNC_COUNT_REG;
  DSI_config->HSYNC_bpc = DSICONTROLLER->HORIZ_BACK_PORCH_COUNT_REG;
  DSI_config->HSYNC_fpc = DSICONTROLLER->HORIZ_FRONT_PORCH_COUNT_REG;
  DSI_config->HSYNC_aac = DSICONTROLLER->HORIZ_ACTIVE_AREA_COUNT_REG;
  DSI_config->VSYNC_count = DSICONTROLLER->VSYNC_COUNT_REG;
  DSI_config->VSYNC_bpc = DSICONTROLLER->VERT_BACK_PORCH_COUNT_REG;
  DSI_config->VSYNC_fpc = DSICONTROLLER->VERT_FRONT_PORCH_COUNT_REG;
  
  DSI_config->polarity = (DSI_polarity)(int)DSICONTROLLER->POLARITY_REG;
  DSI_config->LP_byteclk = GET_DSI_CONTROLLER_MODEL_LP_BYTECLK_REG_BYTECLK(DSICONTROLLER->LP_BYTECLK_REG);
  
  DSI_config->dphy_hs_prep = GET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_PREP(DSICONTROLLER->DPHY_PARAM_REG);
  DSI_config->dphy_hs_zero = GET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_ZERO(DSICONTROLLER->DPHY_PARAM_REG);
  DSI_config->dphy_hs_trail = GET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_TRAIL(DSICONTROLLER->DPHY_PARAM_REG);
  DSI_config->dphy_hs_exit = GET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_EXIT(DSICONTROLLER->DPHY_PARAM_REG);
  DSI_config->clk_hs_prep = GET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_PREP(DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG);
  DSI_config->clk_hs_zero = GET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_ZERO(DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG);
  DSI_config->clk_hs_trail = GET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_TRAIL(DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG);
  DSI_config->clk_hs_exit = GET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_EXIT(DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG);
  DSI_config->lane_high_low_switch_count = GET_DSI_CONTROLLER_MODEL_HIGH_LOW_SWITCH_COUNT_REG_DATA_LANE_SWITCH_COUNT(DSICONTROLLER->HIGH_LOW_SWITCH_COUNT_REG);
  DSI_config->hs_to_lp_switch_count = GET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_HS_TO_LP_SWITCHING_COUNTER(DSICONTROLLER->CLK_LANE_SWT_REG);
  DSI_config->lp_to_hs_switch_count = GET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_LP_TO_HS_SWITCHING_COUNTER(DSICONTROLLER->CLK_LANE_SWT_REG);
  DSI_config->err_auto_recovery = (DSI_err_auto_rec)(int)DSICONTROLLER->AUTO_ERR_REC_REG;
}

void DSI_cl::fill_reduced_reg_info(void){
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[0] = DSI_CONTROLLER_MODEL_DSICONTROLLER[1];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[1] = DSI_CONTROLLER_MODEL_DSICONTROLLER[4];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[2] = DSI_CONTROLLER_MODEL_DSICONTROLLER[5];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[3] = DSI_CONTROLLER_MODEL_DSICONTROLLER[11];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[4] = DSI_CONTROLLER_MODEL_DSICONTROLLER[12];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[5] = DSI_CONTROLLER_MODEL_DSICONTROLLER[24];
  DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[6] = DSI_CONTROLLER_MODEL_DSICONTROLLER[28];
}

void DSI_cl::set_pll_div_ratio(unsigned int div_ratio){
  unsigned int buf_reg, l_div_ratio;
  
  if (div_ratio < div_ratio_min)
    l_div_ratio = div_ratio_min;
  else if (div_ratio > div_ratio_max)
    l_div_ratio = div_ratio_max;
  else
    l_div_ratio = div_ratio;
  
  buf_reg = DSICONTROLLER->TRIM_REG1;
  buf_reg = SET_DSI_CONTROLLER_MODEL_TRIM_REG1_CNT_A(buf_reg, l_div_ratio >> 1);
  buf_reg = SET_DSI_CONTROLLER_MODEL_TRIM_REG1_CNT_B(buf_reg, l_div_ratio & 0x01);
  DSICONTROLLER->TRIM_REG1 = buf_reg;
}

unsigned int DSI_cl::get_pll_div_ratio(){
  unsigned int buf_reg, div_ratio;
  
  buf_reg = DSICONTROLLER->TRIM_REG1;
  div_ratio = (GET_DSI_CONTROLLER_MODEL_TRIM_REG1_CNT_A(buf_reg) << 1) + GET_DSI_CONTROLLER_MODEL_TRIM_REG1_CNT_B(buf_reg);
  return div_ratio;
}

// p_pclk_freq: 20 to 150MHZ pixel clock
int DSI_cl::set_auto_ddr_clk_freq(float p_pclk_freq){
  int pixel_format = 0, video_mode_format = 0, lane_count = 1;
  int ddr_clk_freq_rez;
  
  pclk_freq = p_pclk_freq;
  t_pclk = (float)1000/p_pclk_freq;
  switch (GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_FORMAT_VIDEO_MODE(DSICONTROLLER->DSI_FUNC_PRG_REG)) {
    case DSI_video_format_RGB565:
      pixel_format = 16;
      break;
    case DSI_video_format_RGB666:
      pixel_format = 18;
      break;
    case DSI_video_format_RGB666_lp:
    case DSI_video_format_RGB888:
      pixel_format = 24;
      break;
  }
  if (GET_DSI_CONTROLLER_MODEL_VIDEO_MODE_FORMAT_REG_VIDEO_MODE_FORMAT(DSICONTROLLER->VIDEO_MODE_FORMAT_REG) == DSI_vd_mode_burst)
    video_mode_format = 2;
  else
    video_mode_format = 1;
  
  lane_count = GET_DSI_CONTROLLER_MODEL_DSI_FUNC_PRG_REG_DATA_LANES_PRG_REG(DSICONTROLLER->DSI_FUNC_PRG_REG);
//   CPPTEST_ASSERT(lane_count > 0);
  ddr_clk_freq = (pclk_freq * pixel_format * video_mode_format) / (2*lane_count);
  
  report_high("ddr_clk_freq:");
  print_val(pixel_format);
  print_val(video_mode_format);
  
  print_val_dec(ddr_clk_freq);
  
  if ((ddr_clk_freq % 12) != 0) {
//     if ((ddr_clk_freq % 12) > 5)
      ddr_clk_freq_rez = (ddr_clk_freq/12+1)*12;
//     else
//       ddr_clk_freq_rez = (ddr_clk_freq/12)*12;
  } else 
      ddr_clk_freq_rez = ddr_clk_freq;
  
  if (ddr_clk_freq_rez > ddr_clk_freq_max) {
    ddr_clk_freq_rez = ddr_clk_freq_max;
    report_error("Error ddr_clk_freq value");
  }
  
  t_byteclk = (float)4000/ddr_clk_freq;
  return ddr_clk_freq_rez;
}

void DSI_cl::set_spec_ddr_clk_freq(int p_ddr_clk_freq){
  ddr_clk_freq = p_ddr_clk_freq;
  t_byteclk = (float)4000/ddr_clk_freq;
}

int DSI_cl::get_ddr_clk_freq(){
  return ddr_clk_freq;
}

void DSI_cl::set_t_byteclk(int p_t_byteclk) {
  t_byteclk = p_t_byteclk;
}

int DSI_cl::get_t_byteclk() {
  return t_byteclk;
}

static int dsi_max(int x, int y) {
  return (x) >= (y) ? (x) : (y);
}

void DSI_cl::set_auto_dphy_timing(){
  //   Note: UI is any DDR half cycle.
  //   UI = 500/ddr_clk_freq;
  int HS_PREP_tbl, HS_ZERO_tbl, HS_TRAIL_tbl, HS_EXIT_tbl = 115;
  int n = 1;	// Where n = 1 for Forward-direction HS mode and n = 4 for Reverse-direction HS mode
  int buf_reg, dln_cnt_hs_prep, dln_cnt_hs_zero, dln_cnt_hs_trail, dln_cnt_hs_exit,
    cln_cnt_prep, cln_cnt_zero, cln_cnt_hs_trail, cln_cnt_hs_exit,
    lp_byteclk, high_ls_count, hs_to_lp_switch_count, lp_to_hs_switch_count;
  
  HS_PREP_tbl = ceil(60 + (float)4*500/ddr_clk_freq);
  HS_ZERO_tbl = ceil(170 + (float)10*500/ddr_clk_freq) - HS_PREP_tbl;
  HS_TRAIL_tbl = dsi_max(ceil((float)n*8*500/ddr_clk_freq), ceil(60 + (float)n*4*500/ddr_clk_freq)) + 30;
  
  dln_cnt_hs_prep =	round((float)abs(HS_PREP_tbl - 9*1000/ddr_clk_freq)/t_byteclk - 1);
  dln_cnt_hs_zero =	ceil((float)HS_ZERO_tbl/t_byteclk - 1);
  dln_cnt_hs_trail =	ceil((float)HS_TRAIL_tbl/t_byteclk - 2);
  dln_cnt_hs_exit =	ceil((float)HS_EXIT_tbl/t_byteclk - 1);
//   dln_cnt_hs_exit =	ceil((float)HS_EXIT_tbl/t_byteclk - 3);
  if (dln_cnt_hs_prep < 0)
    dln_cnt_hs_prep = 0;
  if (dln_cnt_hs_zero < 0)
    dln_cnt_hs_zero = 0;
  if (dln_cnt_hs_trail < 0)
    dln_cnt_hs_trail = 0;
  if (dln_cnt_hs_exit < 0)
    dln_cnt_hs_exit = 0;
  
  int CLK_PREP_tbl = 60, CLK_ZERO_tbl, CLK_TRAIL_tbl = 60, CLK_EXIT_tbl = 60;
  CLK_ZERO_tbl = 330 - CLK_PREP_tbl;
  
  cln_cnt_prep =	ceil((float)CLK_PREP_tbl/t_byteclk - 1);
  cln_cnt_zero =	ceil((float)CLK_ZERO_tbl/t_byteclk - 1);
//   cln_cnt_hs_trail =	ceil(float)(CLK_TRAIL_tbl - 3*1000/ddr_clk_freq);
  cln_cnt_hs_trail =	ceil((float)(CLK_TRAIL_tbl - 3*1000/ddr_clk_freq)/t_byteclk) + 3;
  cln_cnt_hs_exit =	ceil((float)(CLK_EXIT_tbl - 2*1000/ddr_clk_freq)/t_byteclk - 1);
  if (cln_cnt_prep < 0)
    cln_cnt_prep = 0;
  if (cln_cnt_zero < 0)
    cln_cnt_zero = 0;
  if (cln_cnt_hs_trail < 0)
    cln_cnt_hs_trail = 0;
  if (cln_cnt_hs_exit < 0)
    cln_cnt_hs_exit = 0;
  
  lp_byteclk = ceil((float)ddr_clk_freq/(12*4));
  high_ls_count = 4*lp_byteclk + dln_cnt_hs_prep + dln_cnt_hs_zero + 4*(int)t_byteclk;
  hs_to_lp_switch_count = cln_cnt_hs_trail + cln_cnt_hs_exit + 3*(int)t_byteclk;
  lp_to_hs_switch_count = 4*lp_byteclk + cln_cnt_prep + cln_cnt_zero + (int)ceil((float)8*500/ddr_clk_freq) + 4*(int)t_byteclk;
  
  buf_reg = DSICONTROLLER->DPHY_PARAM_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_PREP(buf_reg, dln_cnt_hs_prep);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_ZERO(buf_reg, dln_cnt_hs_zero);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_TRAIL(buf_reg, dln_cnt_hs_trail);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPHY_PARAM_REG_HS_EXIT(buf_reg, dln_cnt_hs_exit);
  DSICONTROLLER->DPHY_PARAM_REG = buf_reg;
  
  buf_reg = DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_PREP(buf_reg, cln_cnt_prep);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_ZERO(buf_reg, cln_cnt_zero);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_TRAIL(buf_reg, cln_cnt_hs_trail);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_TIMING_PARAM_REG_HS_EXIT(buf_reg, cln_cnt_hs_exit);
  DSICONTROLLER->CLK_LANE_TIMING_PARAM_REG = buf_reg;
  
  DSICONTROLLER->LP_BYTECLK_REG = SET_DSI_CONTROLLER_MODEL_LP_BYTECLK_REG_BYTECLK(DSICONTROLLER->LP_BYTECLK_REG, lp_byteclk);
  DSICONTROLLER->HIGH_LOW_SWITCH_COUNT_REG = SET_DSI_CONTROLLER_MODEL_HIGH_LOW_SWITCH_COUNT_REG_DATA_LANE_SWITCH_COUNT(DSICONTROLLER->HIGH_LOW_SWITCH_COUNT_REG, high_ls_count);
  
  buf_reg = DSICONTROLLER->CLK_LANE_SWT_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_HS_TO_LP_SWITCHING_COUNTER(buf_reg, hs_to_lp_switch_count);
  buf_reg = SET_DSI_CONTROLLER_MODEL_CLK_LANE_SWT_REG_LP_TO_HS_SWITCHING_COUNTER(buf_reg, lp_to_hs_switch_count);
  DSICONTROLLER->CLK_LANE_SWT_REG = buf_reg;
  
  report_high("set_auto_dphy_timing:");
  print_val(dln_cnt_hs_prep);
  print_val(dln_cnt_hs_zero);
  print_val(dln_cnt_hs_trail);
  print_val(dln_cnt_hs_exit);
  
  print_val(cln_cnt_prep);
  print_val(cln_cnt_zero);
  print_val(cln_cnt_hs_trail);
  print_val(cln_cnt_hs_exit);
  
  print_val(lp_byteclk);
  print_val(high_ls_count);
  print_val(hs_to_lp_switch_count);
  print_val(lp_to_hs_switch_count);
}

void DSI_cl::set_dpi_resolution(/*DSI_config_type*/int *DSI_dpi_resolution) {
  DSI_config_type *l_DSI_dpi_resolution;
  int buf_reg;
  l_DSI_dpi_resolution = (DSI_config_type *)DSI_dpi_resolution;
  
  buf_reg = DSICONTROLLER->DPI_RESOLUTION_REG;
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_HOR_ADDRESS_COUNT(buf_reg, l_DSI_dpi_resolution->DPI_resolution_h);
  buf_reg = SET_DSI_CONTROLLER_MODEL_DPI_RESOLUTION_REG_VERT_ADDRESS_COUNT(buf_reg, l_DSI_dpi_resolution->DPI_resolution_v);
  DSICONTROLLER->DPI_RESOLUTION_REG = buf_reg;
  
  DSICONTROLLER->HSYNC_COUNT_REG = ceil((float)l_DSI_dpi_resolution->HSYNC_count*t_pclk/t_byteclk);
  DSICONTROLLER->HORIZ_BACK_PORCH_COUNT_REG = dsi_max(ceil((float)l_DSI_dpi_resolution->HSYNC_bpc*t_pclk/t_byteclk), HSYNC_bpc_min);
  DSICONTROLLER->HORIZ_FRONT_PORCH_COUNT_REG = dsi_max(ceil((float)l_DSI_dpi_resolution->HSYNC_fpc*t_pclk/t_byteclk), HSYNC_fpc_min);
  DSICONTROLLER->HORIZ_ACTIVE_AREA_COUNT_REG = ceil((float)l_DSI_dpi_resolution->HSYNC_aac*t_pclk/t_byteclk);
  
  DSICONTROLLER->VSYNC_COUNT_REG = l_DSI_dpi_resolution->VSYNC_count;
  DSICONTROLLER->VERT_BACK_PORCH_COUNT_REG = dsi_max(l_DSI_dpi_resolution->VSYNC_bpc, VSYNC_bpc_min);
  DSICONTROLLER->VERT_FRONT_PORCH_COUNT_REG = dsi_max(l_DSI_dpi_resolution->VSYNC_fpc, VSYNC_fpc_min);
}

void DSI_cl::set_low_power_state(/*dsi_ulps_type*/int dsi_ulps) {
  DSICONTROLLER->DEVICE_READY_REG = SET_DSI_CONTROLLER_MODEL_DEVICE_READY_REG_ULPS_STATE(DSICONTROLLER->DEVICE_READY_REG, dsi_ulps);
}

/*dsi_ulps_type*/int DSI_cl::get_low_power_state() {
  return GET_DSI_CONTROLLER_MODEL_DEVICE_READY_REG_ULPS_STATE(DSICONTROLLER->DEVICE_READY_REG);
}