#ifndef DSI_MISC_H
#define DSI_MISC_H

#include "common_structs_and_funct.hpp"
#include "dsi_fields.hpp"
#include "dsi_map.hpp"
#include "dsi_reg_info.hpp"

#define TOTAL_CNT_DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED 7
extern RegInfo DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED[TOTAL_CNT_DSI_CONTROLLER_MODEL_DSICONTROLLER_REDUCED];

#define DSI_LCD_IRQ_BIT	32 + 38

/*! 
*@file dsi_misc.hpp
*/

///@defgroup function function
/*!
* @defgroup Group_DSI For DSI
* @ingroup function
*/

/*!
* @ingroup Group_DSI
*  DSI interrupr request type
* @see DSI_cl::maskIrq(), DSI_cl::unmaskIrq(),
* DSI_cl::isIrqMasked()
*/
enum DSI_irq_type {
    DSI_irq_None 		= 0x00000000,
    DSI_irq_RxSotErr 		= 0x00000001,
    DSI_irq_RxSotSyncErr 	= 0x00000002,
    DSI_irq_RxEotSyncErr 	= 0x00000004,
    DSI_irq_RxEscModeEntrErr 	= 0x00000008,
    DSI_irq_RxLP_TxSyncErr 	= 0x00000010,
    DSI_irq_RxPerphTimeoutErr 	= 0x00000020,
    DSI_irq_RxFalseCntrlErr 	= 0x00000040,
    DSI_irq_RxECC_SnglBitErr 	= 0x00000080,
    DSI_irq_RxECC_MultBitErr 	= 0x00000100,
    DSI_irq_RxChckSmErr 	= 0x00000200,
    DSI_irq_RxDSI_NoDataType 	= 0x00000400,
    DSI_irq_RxDSI_VCID_inval	= 0x00000800,
    DSI_irq_TxFalseCntrlErr	= 0x00001000,
    DSI_irq_TxECC_SnglBitErr	= 0x00002000,
    DSI_irq_TxECC_MultBitErr 	= 0x00004000,
    DSI_irq_TxChckSmErr 	= 0x00008000,
    DSI_irq_TxDSI_NoDataType 	= 0x00010000,
    DSI_irq_TxDSI_VCID_inval	= 0x00020000,
    DSI_irq_HghCont		= 0x00040000,
    DSI_irq_LowCont		= 0x00080000,
    DSI_irq_FifoEmpty		= 0x00100000,
    DSI_irq_HS_TxTimeout	= 0x00200000,
    DSI_irq_LP_RxTimeout	= 0x00400000,
    DSI_irq_AckTimeout		= 0x00800000,
    DSI_irq_AckNoErr		= 0x01000000,
    DSI_irq_TransCntErr		= 0x02000000,
    DSI_irq_ProtocolErr		= 0x04000000,
    DSI_irq_SpclPackCmdSent	= 0x08000000,
    DSI_irq_DBI_FIFO_Full	= 0x10000000,
    DSI_irq_RxContDetect	= 0x20000000,
    DSI_irq_all			= 0x3FFFFFFF
};

/*!
* @ingroup Group_DSI
*  Number of data lanes to be supported is programmed
*  by the processor.
* @see DSI_config_type
*/
enum DSI_DataLines {
  DSI_DataLines_1 = 1,
  DSI_DataLines_2 = 2,
  DSI_DataLines_3 = 3,
  DSI_DataLines_4 = 4
};

/*!
* @ingroup Group_DSI
*  Virtual channel number is programmed by the processor
* @see DSI_config_type
*/
enum DSI_virt_ch {
  DSI_virt_ch_0 = 0,	// Virtual channel 0
  DSI_virt_ch_1 = 1,	// Virtual channel 1
  DSI_virt_ch_2 = 2,	// Virtual channel 2
  DSI_virt_ch_3 = 3	// Virtual channel 3
};

/*!
* @ingroup Group_DSI
*  Supported colour format for video mode.
* @see DSI_config_type
*/
enum DSI_video_format {
  DSI_video_format_none = 0,		// Video mode is not supported.
  DSI_video_format_RGB565 = 1,		// 16-bit format
  DSI_video_format_RGB666 = 2,		// 18-bit format
  DSI_video_format_RGB666_lp = 3,	// RGB666 loosely packed format
  DSI_video_format_RGB888 = 4		// 24-bit format
};

/*!
* @ingroup Group_DSI
*  Supported data width in command mode.
* @see DSI_config_type
*/
enum DSI_cmd_data_width {
  DSI_cmd_data_none = 0,	// Command mode is not supported
  DSI_cmd_data_16bit = 1,	// 16 bit data
  DSI_cmd_data_9bit = 2,	// 9 bit data
  DSI_cmd_data_8bit = 3 	// 8 bit data
};

/*!
* @ingroup Group_DSI
*  Video mode format (packet sequence) to be supported in DSI.
* @see DSI_config_type
*/
enum DSI_video_mode {
  DSI_vd_mode_reserved = 0,
  DSI_vd_mode_non_burst_sync_pulse = 1,
  DSI_vd_mode_non_burst_sync_events = 2,
  DSI_vd_mode_burst = 3
};

/*!
* @ingroup Group_DSI
*  Set by the processor to enable or disable EOT short packet
*  transmission
* @see DSI_config_type
*/
enum DSI_clk_eot {
  DSI_clk_eot_none = 0,
  DSI_clk_eot_disable = 1,
  DSI_clk_eot_clock_stopping = 2,
  DSI_clk_eot_BTA_disable = 4,
};

/*!
* @ingroup Group_DSI
*  Polarity values
* @see DSI_config_type
*/
enum DSI_polarity {
  DSI_polarity_none = 0,
  DSI_polarity_VSYNC = 1,
  DSI_polarity_HSYNC = 2,
  DSI_polarity_shut_down = 4,
  DSI_polarity_color_mode = 8
};

/*!
* @ingroup Group_DSI
*  Error auto recovery values
* @see DSI_config_type
*/
enum DSI_err_auto_rec {
  DSI_err_auto_rec_none = 0x00,
  DSI_err_auto_rec_ecc_mul = 0x01,
  DSI_err_invld_dt = 0x02,
  DSI_err_hi_cont = 0x04,
  DSI_err_lo_cont = 0x08,
  DSI_err_hs_rx_timeout = 0x10,
  DSI_err_lp_rx_timeout = 0x20
};

/*!
* @ingroup Group_DSI
*  Error auto recovery values
* @see DSI_cl::set_low_power_state()
*/
enum dsi_ulps_type {
  dsi_ulps_off	= 0,
  dsi_ulps_exit	= 1,
  dsi_ulps_on	= 2
};

/*!
* @ingroup Group_DSI
*  This structure is used to set configuration DSI parameters.
* @note To initialize the structure, should call
*  the get_config() function.
* @see DSI_cl::set_config(), DSI_cl::get_config(), DSI_cl::set_dpi_resolution()
*/
struct DSI_config_type {
  DSI_DataLines 	data_lines;
  DSI_virt_ch 		ch_video_mode;
  DSI_virt_ch 		ch_cmd_mode;
  DSI_video_format 	video_format;
  DSI_cmd_data_width	cmd_data_width;
  
  int hs_tx_timeout;
  int lp_rx_timeout;
  int turn_around_timeout;
  int dev_reset_timer;
  int init_counter;
  
  DSI_video_mode video_mode;
  DSI_clk_eot clk_eot_reg;
  int DPI_resolution_h;
  int DPI_resolution_v;
  
  int HSYNC_count;
  int HSYNC_bpc;
  int HSYNC_fpc;
  int HSYNC_aac;
  int VSYNC_count;
  int VSYNC_bpc;
  int VSYNC_fpc;
  
  int polarity;
  int LP_byteclk;
  int dphy_hs_prep;
  int dphy_hs_zero;
  int dphy_hs_trail;
  int dphy_hs_exit;
  int clk_hs_prep;
  int clk_hs_zero;
  int clk_hs_trail;
  int clk_hs_exit;
  int lane_high_low_switch_count;
  int hs_to_lp_switch_count;
  int lp_to_hs_switch_count;
  DSI_err_auto_rec err_auto_recovery;
};

/*!
* @ingroup Group_DSI
*  Maximum value ​​for the PLL divider ratio
* @see DSI_cl::set_pll_div_ratio()
*/
#define div_ratio_max	41
/*!
* @ingroup Group_DSI
*  Minimum value ​​for the PLL divider ratio
* @see DSI_cl::set_pll_div_ratio()
*/
#define div_ratio_min	 4

/*!
* @ingroup Group_DSI
*  Minimum value ​​for the Horizontal Back Porch (HBP)
* @see DSI_cl::set_config(), DSI_cl::get_config()
*/
#define HSYNC_bpc_min	 16
/*!
* @ingroup Group_DSI
*  Minimum value ​​for the Horizontal Front Porch (HFP)
* @see DSI_cl::set_config(), DSI_cl::get_config()
*/
#define HSYNC_fpc_min	 16

/*!
* @ingroup Group_DSI
*  Minimum value ​​for the Vertical Back Porch (VBP)
* @see DSI_cl::set_config(), DSI_cl::get_config()
*/
#define VSYNC_bpc_min	 2
/*!
* @ingroup Group_DSI
*  Minimum value ​​for the Vertical Front Porch (VFP)
* @see DSI_cl::set_config(), DSI_cl::get_config()
*/
#define VSYNC_fpc_min	 2

/*!
* @ingroup Group_DSI
*  Maximum value ​​for the ddr_clk in MHz
* @see DSI_cl::set_auto_ddr_clk_freq()
*/
#define ddr_clk_freq_max 492

class DSI_cl : public DSI_CONTROLLER_MODEL_BASE {
  private:
    int ddr_clk_freq;
    float t_pclk, t_byteclk, pclk_freq;
    //int irq_bit_n;
  public:

    /*!
    * @ingroup Group_DSI
    *  Enable DSI
    * @see disable()
    */
    void enable();
    
    /*!
    * @ingroup Group_DSI
    *  Disable DSI
    * @see enable()
    */
    void disable();
    
    /*!
    * @ingroup Group_DSI
    *  Enable DFE
    * @see disable_DFE()
    */
    void enable_DFE();
    
    /*!
    * @ingroup Group_DSI
    *  Disable DFE
    * @see enable_DFE()
    */
    void disable_DFE();
    
    /*!
    * @ingroup Group_DSI
    *  The function returns the mapping DSI interrupts
    * in the interrupt controller MPU
    * @return irq bit number
    * @see maskIrq()
    */
    int get_irq_bit();
    
    /*!
    * @ingroup Group_DSI
    *  Enable DSI interrupts
    * @param dsi_irq - DSI interrupt request type
    * @see unmaskIrq(), DSI_irq_type
    */
    void maskIrq(/*DSI_irq_type*/ int dsi_irq);
    
    /*!
    * @ingroup Group_DSI
    *  Disable DSI interrupts
    * @param dsi_irq - DSI interrupt request type
    * @see maskIrq(), DSI_irq_type
    */
    void unmaskIrq(/*DSI_irq_type*/ int dsi_irq);
    
    /*!
    * @ingroup Group_DSI
    *  Check whether the selected interrupt
    * @param dsi_irq - DSI interrupt request type
    * @return true - if the interrupt is masked
    * @return false - if the interrupt is NOT masked
    * @see maskIrq(), DSI_irq_type
    */
    bool isIrqMasked(/*DSI_irq_type*/ int dsi_irq);
    
    /*!
    * @ingroup Group_DSI
    *  Clear DSI interrupts
    * @param dsi_irq - DSI interrupt request type
    * @see maskIrq(), unmaskIrq(), DSI_irq_type
    */
    void clearIrq(/*DSI_irq_type*/ int dsi_irq);
    
    /*!
    * @ingroup Group_DSI
    *  Check whether the masked interrupt active
    * @param dsi_irq - DSI interrupt request type
    * @see maskIrq(), clearIrq(), DSI_irq_type
    */
    bool isIrqActive(/*DSI_irq_type*/ int dsi_irq);
    
    /*!
    * @ingroup Group_DSI
    *  Returns Interuppt status register
    * @return Interuppt status register value
    * @see maskIrq(), clearIrq(), DSI_irq_type
    */
    unsigned int getIrqActive();
    
    /*!
    * @ingroup Group_DSI
    *  Set DSI configure parameters
    * @param DSI_config - configuration structure handle
    * @see set_reduce_config(), get_config(), DSI_config_type
    */
    void set_config(/*DSI_config_type*/int *DSI_config_p);
    
    /*!
    * @ingroup Group_DSI
    *  Set DSI configure parameters in the reduced version
    * @param DSI_config - configuration structure handle
    * The function uses only the DSI_config_type structure fields:
    * 	- DSI_DataLines data_lines;
    * 	- DSI_virt_ch ch_video_mode;
    * 	- DSI_virt_ch ch_cmd_mode;
    * 	- DSI_video_format video_format;
    * 	- DSI_cmd_data_width cmd_data_width;
    * 	- int hs_tx_timeout;
    * 	- int lp_rx_timeout;
    * 	- int turn_around_timeout;
    * 	- int dev_reset_timer;
    * 	- int init_counter;
    * 	- DSI_video_mode video_mode;
    * 	- DSI_clk_eot clk_eot_reg;
    * 	- int polarity;
    * 	- DSI_err_auto_rec err_auto_recovery;
    * @see set_config(), get_config(), DSI_config_type
    */
    void set_reduce_config(/*DSI_config_type*/int *DSI_config_p);
    
    /*!
    * @ingroup Group_DSI
    *  Get DSI configure parameters
    * @param DSI_config - configuration structure handle
    * @see set_config(), set_reduce_config(), DSI_config_type
    */
    void get_config(/*DSI_config_type*/int *DSI_config_p);
    
    /*!
    * @ingroup Group_DSI
    * generate reduced RegInfo vector for fast register tests
    */
    void fill_reduced_reg_info();
    
    /*!
    * @ingroup Group_DSI
    *  The PLL uses a Dual-modulus divider. The equation to
    * compute the divider ratio is N = 2*A + B, where A is the
    * cnt_a value (6 bits) and B is the cnt_b value (1 bit).
    * @param div_ratio - division ratio
    * @see get_pll_div_ratio(), div_ratio_min, div_ratio_max
    */
    void set_pll_div_ratio(unsigned int div_ratio);
    
    /*!
    * @ingroup Group_DSI
    *  The PLL uses a Dual-modulus divider. The equation to
    * compute the divider ratio is N = 2*A + B, where A is the
    * cnt_a value (6 bits) and B is the cnt_b value (1 bit).
    * @return Current set division ratio
    * @see set_pll_div_ratio(), div_ratio_min, div_ratio_max
    */
    unsigned int get_pll_div_ratio();
    
    /*!
    * @ingroup Group_DSI
    *  Automatic calculate DDR clock frequency.
    * @param p_pclk_freq - 20 to 150MHZ pixel clock frequency.
    * @return The result of calculation DDR clock frequency
    * @note This function must be called after set_config()
    * @see get_ddr_clk_freq(), set_spec_ddr_clk_freq(), set_config()
    */
    int set_auto_ddr_clk_freq(float p_pclk_freq);
    
    /*!
    * @ingroup Group_DSI
    *  Set a special value of DDR clock frequency.
    * @param p_ddr_clk_freq - value of DDR clock frequency in MHz
    * @see get_ddr_clk_freq(), set_auto_ddr_clk_freq()
    */
    void set_spec_ddr_clk_freq(int p_ddr_clk_freq);
    
    /*!
    * @ingroup Group_DSI
    *  Return DDR clock frequency.
    * @return DDR clock frequency in MHz
    * @see set_auto_ddr_clk_freq(), set_spec_ddr_clk_freq()
    */
    int get_ddr_clk_freq();
    
    /*!
    * @ingroup Group_DSI
    *  Set a special value of High-Speed Transmit Byte Clock.
    * @param p_t_byteclk - value of t_byteclk time in ns
    * @see get_t_byteclk(), set_auto_ddr_clk_freq()
    */
    void set_t_byteclk(int p_t_byteclk);
    
    /*!
    * @ingroup Group_DSI
    *  Return t_byteclk time in ns.
    * @note This parameter is set in the functions set_auto_ddr_clk_freq()
    * or set_spec_ddr_clk_freq()
    * @return t_byteclk time
    * @see get_ddr_clk_freq(), set_auto_ddr_clk_freq()
    */
    int get_t_byteclk();
    
    /*!
    * @ingroup Group_DSI
    *  Automatic calculate DPHY timing.
    * @note This function must be called after set_config() and
    * set_auto_ddr_clk_freq()
    * @see set_auto_ddr_clk_freq(), set_config()
    */
    void set_auto_dphy_timing();
    
    /*!
    * @ingroup Group_DSI
    *  Automatic calculate DPI resolution parameters.
    * @note This function must be called after set_auto_ddr_clk_freq()
    * @param DSI_dpi_resolution - configuration structure handle.
    * The function uses only the DSI_config_type structure fields:
    * 	- int DPI_resolution_h;
    * 	- int DPI_resolution_v;
    * 	- int HSYNC_count;
    * 	- int HSYNC_bpc;
    * 	- int HSYNC_fpc;
    * 	- int HSYNC_aac;
    * 	- int VSYNC_count;
    * 	- int VSYNC_bpc;
    * 	- int VSYNC_fpc;
    * @see set_config(), DSI_config_type
    */
    void set_dpi_resolution(/*DSI_config_type*/int *DSI_dpi_resolution);
    
    /*!
    * @ingroup Group_DSI
    *  Set the ultra low power mode of DSI
    * @param dsi_ulps - ultra low power mode
    * @see dsi_ulps_type
    */
    void set_low_power_state(/*dsi_ulps_type*/int dsi_ulps);
    
    /*!
    * @ingroup Group_DSI
    *  Returns the current ultra low power mode of DSI
    * @return ultra low power mode
    * @see dsi_ulps_type
    */
    /*dsi_ulps_type*/int get_low_power_state();
};
#endif