#ifndef VPOUT_CH_MISC_H
#define VPOUT_CH_MISC_H

#include <string.h>

#include "common_structs_and_funct.hpp"
#include "vpout_reg_fields.hpp"
#include "vpout_reg_map.hpp"
#include "vpout_reg_info.hpp"
#include "tracer.hpp"

#define TOTAL_CNT_LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED 6
extern RegInfo LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[TOTAL_CNT_LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED];

/*! 
*@file vpout_misc.hpp
*/

///@defgroup function function
/*!
* @defgroup Group_VPOUT For VPOUT
* @ingroup function
*/

/*!
* @ingroup Group_VPOUT
*  Format of the input data.
*/
enum vpout_format_indata {
  vpout_in_Color_8bpp  = 0,	// 8 bits color
  vpout_in_Color_12bpp = 1,	// 12 bits color
  vpout_in_Color_15bpp = 2,	// 15 bits color
  vpout_in_Color_16bpp = 3,	// 16 bits color
  vpout_in_Color_18bpp = 4,	// 18 bits color
  vpout_in_Color_24bpp = 5,	// 24 bits color
  vpout_in_Color_32bpp = 6,	// 32 bits color
  vpout_in_Reserved    = 7,	// reserved
  vpout_in_Index_1bpp  = 8,	// 1 bit index palette
  vpout_in_Index_2bpp  = 9,	// 2 bits index palette
  vpout_in_Index_4bpp  = 10,	// 4 bits index palette
  vpout_in_Index_8bpp  = 11	// 8 bits index palette
};

/*!
* @ingroup Group_VPOUT
*  Hardware cursor on/off.
*/
enum vpout_hw_cursor {
  vpout_cursor_off = 0,
  vpout_cursor_on = 1
};

/*!
* @ingroup Group_VPOUT
*  Hardware cursor size.
*/
enum vpout_hw_cursor_size {
  vpout_cursor_32x32 = 0,
  vpout_cursor_64x64 = 1
};

/*!
* @ingroup Group_VPOUT
*  Number of buffers used.
*/
enum vpout_buffering_mode {
  vpout_one_buffering = 0,
  vpout_double_buffering = 1
};

/*!
* @ingroup Group_VPOUT
*  Interrupts VPOUT.
*/
enum vpout_int_def {
  vpout_int_DMA_DONE		= 0x01,
  vpout_int_DMA_FIFO_EMPTY	= 0x02,
  vpout_int_OUT_FIFO_INT	= 0x04,
  vpout_int_OUT_FIFO_EMPTY	= 0x08,
  vpout_int_DMA_READ_ERROR	= 0x10,
  vpout_int_SYNC_DONE		= 0x20
};

/*!
* @ingroup Group_VPOUT
*  Index cursor memory.
*/
enum vpout_cur_color_ind {
  vpout_cur_color0	 = 2,
  vpout_cur_color1	 = 3,
  vpout_cur_color_clear	 = 0,
  vpout_cur_color_clear2 = 1
};

/*!
* @ingroup Group_VPOUT
*  DSI_CTR register fields.
*/
enum vpout_dsi_ctr {
  vpout_dsi_ctr_sd	 = 0x01,	// Shutdown [DPI]
  vpout_dsi_ctr_cm	 = 0x02,	// Color mode [DPI]
  
  vpout_dsi_ctr_gm	 = 0x04,	// Generic mode [DBI]
  vpout_dsi_ctr_lp	 = 0x08,	// Low power mode [DBI]
  vpout_dsi_ctr_gvn	 = 0x30		// Virtual channel number for generic transfers [DBI]
};

/*!
* @ingroup Group_VPOUT
*  Structure configuration VPOUT.
*/
struct vpout_config_type {
  int hgate;
  int vgate;
  int hsw;
  int hgdel;
  int hlen;
  int vsw;
  int vgdel;
  int vlen;
  int numb_of_frames;
};

/*!
* @ingroup Group_VPOUT
*  Description header of the cursor file.
*/
struct cur_header_type {
  short idReserved;
  short idType;
  short idCount;
} __attribute__((__packed__));

/*!
* @ingroup Group_VPOUT
*  Description of the cursor image header.
*/
struct cur_entry_header_type {
  char bWidth;
  char bHeight;
  char bColorCount;
  char bReserved;
  short wPlanes;
  short wBitCount;
  long int dwBytesInRes;
  long int dwImageOffset;
} __attribute__((__packed__));

/*!
* @ingroup Group_VPOUT
*  Description header of the image file.
*/
struct bitmap_file_header_type {
  short bfType;		// смещение 0 байт от начала файла
  long int bfSize;	// смещение 2 байта от начала файла, длина 4 байта
  short bfReserved1;
  short bfReserved2;
  long int bfOffBits;	// смещение 10 байт от начала файла, длина 4 байта
} __attribute__((__packed__));

/*!
* @ingroup Group_VPOUT
*  Description of the image header.
*/
struct bitmap_info_header_type {
  long int biSize;
  long int biWidth;
  long int biHeight;
  short biPlanes;
  short biBitCount;
  long int biCompression;
  long int biSizeImage;
  long int biXPelsPerMeter;
  long int biYPelsPerMeter;
  long int biClrUsed;
  long int biClrImportant;
} __attribute__((__packed__));


class VPOUT_cl : public LCD_CONTROLLER_MODEL_BASE {
  private:
    int irq_bit_n;
    bool clk_always_on_enable;
    
  public:
    
    /*!
    * @ingroup Group_VPOUT
    *  Configure vpout
    *  @param ab0_addr - frame buffer address 0
    *  @param ab1_addr - frame buffer address 1
    *  @param hsw - horizontal sync pulse width (HSW)
    *  @param hgdel - horizontal gate delay width (HGDEL)
    *  @param hgate - horizontal gate width (HGATE)
    *  @param hlen - total horizontal line time (HLEN)
    *  @param vsw - vertical sync pulse width (VSW)
    *  @param vgdel - vertical gate delay width (VGDEL)
    *  @param vgate - vertical gate delay time (VGATE)
    *  @param vlen - total vertical frame time (VLEN)
    *  @param DIV - system frequency division ratio
    */
    void configure_base(int ab0_addr, int ab1_addr, int hsw, int hgdel, int hgate, int hlen,
			int vsw, int vgdel, int vgate, int vlen, int DIV);
		      
    /*!
    * @ingroup Group_VPOUT
    *  Set mode for vpout
    *  @param INSIZE - format of the input data:
    *  @param HWCEN - enable hardware cursor:
    *    - vpout_cursor_off = 0 - disable
    *    - vpout_cursor_on = 1 - enable
    *  @param HWC_MODE - hardware cursor mode
    *  @param BUF_MODE - enabling two buffers
    *  @param BUF_NUMB - number of the initial buffer
    *  @param CCM - formation output (without inserting 0 / inserting 0)
    *  @param HINV - invert HSYNC
    *  @param VINV - invert VSYNC
    *  @param DINV - invert DEN
    *  @param PINV - invert PIXCLK
    *  @param INSYNC - external synchronization
    *  @param IN_DEN_EN - allow external signal DEN
    *  @param HDEF - HSYNC signal after switching off VPOUT
    *  @param VDEF - VSYNC signal after switching off VPOUT
    *  @param CLK_ALWAYS_ON - PIXCLK signal works after switching off VPOUT
    */
    void set_mode(/*vpout_format_indata*/int INSIZE, /*vpout_hw_cursor*/int HWCEN, /*vpout_hw_cursor_size*/int HWC_MODE,
		  /*vpout_buffering_mode*/int BUF_MODE, int BUF_NUMB, int CCM, int HINV, int VINV, int DINV, int PINV,
		  int INSYNC, int IN_DEN_EN, int HDEF, int VDEF, int CLK_ALWAYS_ON);
    /*!
    * @ingroup Group_VPOUT
    *  Setting the offset address of the buffers
    *  @param offset0 - address offset 0
    *  @param offset1 - address offset 1
    */
    void set_offsets(int offset0, int offset1);
    
    /*!
    * @ingroup Group_VPOUT
    * Generate reduced RegInfo vector for fast register tests
    */
    void fill_reduced_reg_info();

    /*!
    * @ingroup Group_VPOUT
    *  Settings the hardware cursor
    *  @param X - X-coordinate of the cursor
    *  @param Y - Y-coordinate of the cursor
    *  @param X_P - X-coordinate binding of the cursor
    *  @param Y_P - Y-coordinate binding of the cursor
    *  @param COLOR0 - color 0 of the cursor
    *  @param COLOR1 - color 1 of the cursor
    */
    void cursor_setup(int X, int Y, int X_P, int Y_P, int COLOR0, int COLOR1);
    
    /*!
    * @ingroup Group_VPOUT
    *  Move the cursor
    *  @param X - X-coordinate of the cursor
    *  @param Y - Y-coordinate of the cursor
    */
    void cursor_move(int X, int Y);
    
    /*!
    * @ingroup Group_VPOUT
    *  Set VPOUT irq bit number in global vector
    * @param bit_n - irq bit number
    * @see get_irq_bit()
    */
    void set_irq_bit(int bit_n);
    
    /*!
    * @ingroup Group_VPOUT
    *  The function returns the mapping VPOUT interrupts
    * in the interrupt controller MPU
    * @return irq bit number
    * @see set_int_mask()
    */
    int get_irq_bit();
    
    /*!
    * @ingroup Group_VPOUT
    *  Enable clk_always_on function
    * @param clk_always_on_en - true - clk_always_on enable; false - clk_always_on disable
    */
    void set_clk_always_on_enable(bool clk_always_on_en);
    
    /*!
    * @ingroup Group_VPOUT
    *  Setting interrupt mask
    * @param int_mask - interrupt mask
    * @see get_int_mask(), is_int_active(), clear_int()
    * 
    */
    void set_int_mask(/*vpout_int_def*/int int_mask);
    
    /*!
    * @ingroup Group_VPOUT
    *  The function returns the interrupt mask
    * @return interrupt mask
    * @see set_int_mask(), clear_int()
    */
    /*vpout_int_def*/int get_int_mask();
    
    /*!
    * @ingroup Group_VPOUT
    *  Clear interrupt request
    * @param int_mask - interrupt mask
    * @see set_int_mask(), is_int_active()
    */
    void clear_int(/*vpout_int_def*/int int_mask);
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns whether the specified interrupt is active after masking
    * @return true - if the interrupt is active
    * @return false - if the interrupt is not active
    * @param int_mask - interrupt mask
    * @see set_int_mask(), clear_int()
    */
    bool is_int_active(/*vpout_int_def*/int int_mask);
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns Interuppt status register
    * @return Interuppt status register values
    * @see set_int_mask(), clear_int()
    */
    unsigned int get_int_state();
    
    /*!
    * @ingroup Group_VPOUT
    *  Counter loaded from memory via DMA pixels
    * @param dma_x - pixel counter
    * @param dma_y - line counter
    */
    void get_dmaxy(int* dma_x, int* dma_y);
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns the current loaded via DMA memory address
    *  @return Current address in memory
    */
    int get_dma_address();
    
    /*!
    * @ingroup Group_VPOUT
    *  Enable vpout
    * @note The function calls prog_reset()
    * @see is_enabled(), disable(), start(), prog_reset()
    */
    void enable();
    
    /*!
    * @ingroup Group_VPOUT
    *  The function returns the current state VPOUT
    * @return true if VPOUT enabled, false if disabled
    * @see disable(), start()
    */
    bool is_enabled();
    
    /*!
    * @ingroup Group_VPOUT
    *  Disable vpout
    * @see enable(), start()
    */
    void disable();
    
    /*!
    * @ingroup Group_VPOUT
    *  Sets the flag initialization
    */
    void set_init();
    
    /*!
    * @ingroup Group_VPOUT
    *  Software reset
    * @see enable(), start()
    */
    void prog_reset();
    
    /*!
    * @ingroup Group_VPOUT
    *  Sets the flag initialization then start VPOUT
    * @see stop(), set_init(), wait_numb_frame()
    */
    void start();
    
    /*!
    * @ingroup Group_VPOUT
    *  Stop VPOUT
    * @see start()
    */
    void stop();
    
    /*!
    * @ingroup Group_VPOUT
    *  VPOUT stop and switch off at the end of the frame
    * @param do_clear - enabling a full reset of the VPOUT
    * @see stop(), disable()
    */
    void switch_off(bool do_clear);
    
    /*!
    * @ingroup Group_VPOUT
    *  Waiting for transfer a specified number of frames VPOUT
    * @param numb_frame - specified number of frames
    * @param print_info - enabling the current print position DMA
    * @note The function uses an interrupt vpout_int_DMA_DONE,
    *  which must be initialized before
    * @see start(), set_int_mask()
    */
    void wait_numb_frame(int numb_frame, bool print_info);
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns the base address registers VPOUT
    * @see get_palette_mem_addr(), get_cursor_mem_addr()
    */
    int get_regs_mem_addr();
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns the base address of the palette VPOUT
    * @see get_regs_mem_addr(), get_cursor_mem_addr()
    */
    int get_palette_mem_addr();
    
    /*!
    * @ingroup Group_VPOUT
    *  Returns the base address cursor VPOUT
    * @see get_regs_mem_addr(), get_palette_mem_addr()
    */
    int get_cursor_mem_addr();
    
    /*!
    * @ingroup Group_VPOUT
    *  write 1 byte in memory of the cursor VPOUT
    * @param offset - offset from the beginning of memory
    * @param data - data to be written
    * @see set_cursor_mem_dword()
    */
    void set_cursor_mem_byte(int offset, char data);
    
    /*!
    * @ingroup Group_VPOUT
    *  write 1 dword (4 bytes) in memory of the cursor VPOUT
    * @param offset - offset from the beginning of memory
    * @param data - data to be written
    * @see set_cursor_mem_dword()
    */
    void set_cursor_mem_dword(int offset, int data);
    
    /*!
    * @ingroup Group_VPOUT
    *  Loading cursor file to the hardware cursor memory.
    * @param cursor_file - pointer to a cursor file
    * @see set_cursor_mem_dword(), set_cursor_mem_byte()
    */
    void load_cursor_file(unsigned int *cursor_file);
    
    /*!
    * @ingroup Group_VPOUT
    *  sets the value of the register DSI_CTR
    * @param dsi_ctr - value of the DSI_CTR
    */
    void set_dsi_ctr(/*vpout_dsi_ctr*/int dsi_ctr);
    
};

#endif