#include "vpout_misc.hpp"
#include "vpout_reg_info.cpp"

RegInfo LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[TOTAL_CNT_LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED];

void VPOUT_cl::enable(){
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_EN(LCDCONTROLLER->LCDCSR, 1);
}

bool VPOUT_cl::is_enabled(){
  return GET_LCD_CONTROLLER_MODEL_LCDCSR_EN(LCDCONTROLLER->LCDCSR);
}

void VPOUT_cl::disable(){
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_EN(LCDCONTROLLER->LCDCSR, 0);
}

void VPOUT_cl::configure_base(int ab0_addr, int ab1_addr, int hsw, int hgdel, int hgate, int hlen,
			      int vsw, int vgdel, int vgate, int vlen, int DIV){
  LCDCONTROLLER->LCDHT0 = LCD_CONTROLLER_MODEL_LCDHT0_HSW(hsw) | LCD_CONTROLLER_MODEL_LCDHT0_HGDEL(hgdel);
  LCDCONTROLLER->LCDHT1 = LCD_CONTROLLER_MODEL_LCDHT1_HGATE(hgate) | LCD_CONTROLLER_MODEL_LCDHT1_HLEN(hlen);
  LCDCONTROLLER->LCDVT0 = LCD_CONTROLLER_MODEL_LCDVT0_VSW(vsw) | LCD_CONTROLLER_MODEL_LCDVT0_VGDEL(vgdel);
  LCDCONTROLLER->LCDVT1 = LCD_CONTROLLER_MODEL_LCDVT1_VGATE(vgate) | LCD_CONTROLLER_MODEL_LCDVT1_VLEN(vlen);
  LCDCONTROLLER->LCDDIV = DIV;
  LCDCONTROLLER->LCDAB0 = ab0_addr;
  LCDCONTROLLER->LCDAB1 = ab1_addr;
}

void VPOUT_cl::set_mode(/*vpout_format_indata*/int INSIZE, /*vpout_hw_cursor*/int HWCEN, /*vpout_hw_cursor_size*/int HWC_MODE,
			/*vpout_buffering_mode*/int BUF_MODE, int BUF_NUMB, int CCM, int HINV, int VINV, int DINV, int PINV,
			int INSYNC, int IN_DEN_EN, int HDEF, int VDEF, int CLK_ALWAYS_ON) {
  unsigned int clk_always_on_fields = 0;
  if (this->clk_always_on_enable)
    clk_always_on_fields = LCD_CONTROLLER_MODEL_LCDMODE_HDEF(HDEF) | LCD_CONTROLLER_MODEL_LCDMODE_VDEF(VDEF) | LCD_CONTROLLER_MODEL_LCDMODE_CLK_ALWAYS_ON(CLK_ALWAYS_ON);
  
  LCDCONTROLLER->LCDMODE = LCD_CONTROLLER_MODEL_LCDMODE_INSIZE(INSIZE) | LCD_CONTROLLER_MODEL_LCDMODE_HWCEN(HWCEN) | LCD_CONTROLLER_MODEL_LCDMODE_HWC_MODE(HWC_MODE) |
    LCD_CONTROLLER_MODEL_LCDMODE_BUF_MODE(BUF_MODE) | LCD_CONTROLLER_MODEL_LCDMODE_BUF_NUMB(BUF_NUMB) | LCD_CONTROLLER_MODEL_LCDMODE_HINV(HINV) |
    LCD_CONTROLLER_MODEL_LCDMODE_VINV(VINV) | LCD_CONTROLLER_MODEL_LCDMODE_DINV(DINV) | LCD_CONTROLLER_MODEL_LCDMODE_PINV(PINV) | LCD_CONTROLLER_MODEL_LCDMODE_CCM(CCM) |
    LCD_CONTROLLER_MODEL_LCDMODE_INSYNC(INSYNC) | LCD_CONTROLLER_MODEL_LCDMODE_IN_DEN_EN(IN_DEN_EN) | clk_always_on_fields;
}

void VPOUT_cl::set_offsets(int offset0, int offset1) {
  LCDCONTROLLER->LCD_OFF0 = offset0;
  LCDCONTROLLER->LCD_OFF1 = offset1;
}

void VPOUT_cl::fill_reduced_reg_info(void){
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[0] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[1];
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[1] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[3];
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[2] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[10];
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[3] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[17];
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[4] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[18];
  LCD_CONTROLLER_MODEL_LCDCONTROLLER_REDUCED[5] = LCD_CONTROLLER_MODEL_LCDCONTROLLER[19];
}

void VPOUT_cl::cursor_setup(int X, int Y, int X_P, int Y_P, int COLOR0, int COLOR1){
  LCDCONTROLLER->LCDXY = LCD_CONTROLLER_MODEL_LCDXY_XPOS(X) | LCD_CONTROLLER_MODEL_LCDXY_YPOS(Y);
  LCDCONTROLLER->LCDXYP = LCD_CONTROLLER_MODEL_LCDXYP_XPOS(X_P) | LCD_CONTROLLER_MODEL_LCDXYP_YPOS(Y_P);
  LCDCONTROLLER->LCDCOLOR0 = COLOR0;
  LCDCONTROLLER->LCDCOLOR1 = COLOR1;
}

void VPOUT_cl::cursor_move(int X, int Y){
  LCDCONTROLLER->LCDXY = LCD_CONTROLLER_MODEL_LCDXY_XPOS(X) | LCD_CONTROLLER_MODEL_LCDXY_YPOS(Y);
}

void VPOUT_cl::set_irq_bit(int bit_n) {
  this->irq_bit_n = bit_n;
}

int VPOUT_cl::get_irq_bit() {
  return irq_bit_n;
}

void VPOUT_cl::set_clk_always_on_enable(bool clk_always_on_en) {
  this->clk_always_on_enable = clk_always_on_en;
}

void VPOUT_cl::set_int_mask(/*vpout_int_def*/int int_mask){
  LCDCONTROLLER->LCDINTMASK = SET_LCD_CONTROLLER_MODEL_LCDINTMASK_MASK(LCDCONTROLLER->LCDINTMASK, int_mask);
}

/*vpout_int_def*/int VPOUT_cl::get_int_mask() {
  return LCDCONTROLLER->LCDINTMASK;
}

void VPOUT_cl::clear_int(/*vpout_int_def*/int int_mask){
  LCDCONTROLLER->LCDINT = SET_LCD_CONTROLLER_MODEL_LCDINT_INT(LCDCONTROLLER->LCDINT, int_mask);
}

bool VPOUT_cl::is_int_active(/*vpout_int_def*/int int_mask){
  return LCDCONTROLLER->LCDINT & int_mask;
}

unsigned int VPOUT_cl::get_int_state() {
  return LCDCONTROLLER->LCDINT;
}

void VPOUT_cl::get_dmaxy(int* dma_x, int* dma_y){
  dma_x[0] = GET_LCD_CONTROLLER_MODEL_LCDDMAXY_X(LCDCONTROLLER->LCDDMAXY);
  dma_y[0] = GET_LCD_CONTROLLER_MODEL_LCDDMAXY_Y(LCDCONTROLLER->LCDDMAXY);
}

int VPOUT_cl::get_dma_address(){
  return LCDCONTROLLER->LCDDMAADDRESS;
}

void VPOUT_cl::set_init(){
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_INIT(LCDCONTROLLER->LCDCSR, 1);
}

void VPOUT_cl::prog_reset(){
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_CLR(LCDCONTROLLER->LCDCSR, 1);
  while (GET_LCD_CONTROLLER_MODEL_LCDCSR_CLR(LCDCONTROLLER->LCDCSR));
}

void VPOUT_cl::start(){
  // set init at first
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_INIT(LCDCONTROLLER->LCDCSR, 1);
  while (GET_LCD_CONTROLLER_MODEL_LCDCSR_INIT(LCDCONTROLLER->LCDCSR));
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_RUN(LCDCONTROLLER->LCDCSR, 1);
}
void VPOUT_cl::stop(){
  LCDCONTROLLER->LCDCSR = SET_LCD_CONTROLLER_MODEL_LCDCSR_RUN(LCDCONTROLLER->LCDCSR, 0);
}

void VPOUT_cl::switch_off(bool do_clear){
  this->stop();
//   Ожидание окончания следующего кадра и выключение VPOUT
  this->wait_numb_frame(1, false);
  if (do_clear)
    this->prog_reset();
  else
    this->disable();
}

void VPOUT_cl::wait_numb_frame(int numb_frame, bool print_info) {
  int lcd_dma_x, lcd_dma_y, old_lcd_dma_y;
  
//   Проверка инициализации прерывания vpout_int_SYNC_DONE
  this->set_int_mask(this->get_int_mask() | vpout_int_SYNC_DONE);
  
//   Цикл ожидания передачи заданного числа кадров
  for (int i=0; i<numb_frame; i++) {
    while (!this->is_int_active(vpout_int_SYNC_DONE)) {
      if (print_info) {
	this->get_dmaxy(&lcd_dma_x, &lcd_dma_y);
	if (old_lcd_dma_y != lcd_dma_y) {
	  old_lcd_dma_y = lcd_dma_y;
	  print_val(lcd_dma_y);
	}
      }
    }
    this->clear_int(vpout_int_SYNC_DONE);
  }
}

int VPOUT_cl::get_regs_mem_addr(){
  return (long int)&LCDCONTROLLER->LCDCSR;
}

int VPOUT_cl::get_palette_mem_addr(){
  return (long int)&LCDPALETTEMEM[0];
}

int VPOUT_cl::get_cursor_mem_addr(){
  return (long int)&LCD_CURSOR_MEM[0];
}

void VPOUT_cl::set_cursor_mem_byte(int offset, char data){
  unsigned int get_mem, int_offset, bit_shift_offset;
  
  int_offset = offset/4;
  bit_shift_offset = (offset%4)*8;
  
  get_mem = LCD_CURSOR_MEM[int_offset];
  get_mem &= ~(0xFF << bit_shift_offset);
  get_mem |= data << bit_shift_offset;
  LCD_CURSOR_MEM[int_offset] = get_mem;
}

void VPOUT_cl::set_cursor_mem_dword(int int_offset, int data){
  LCD_CURSOR_MEM[int_offset] = (unsigned int)data;
}

void VPOUT_cl::load_cursor_file(unsigned int *cursor_file) {
  cur_header_type cur_header;
  cur_entry_header_type cur_entry_header;
  bitmap_info_header_type bitmap_info_header;
  
// Чтение заголовка файла курсора
  memcpy(&cur_header, cursor_file, sizeof(cur_header_type));
// Проверка формата файла курсора
  CPPTEST_ASSERT((cur_header.idReserved != 0) || (cur_header.idType != 2));
  
// Чтение заголовка изображения курсора
  int numb_read_curs = 1;	// Первый курсор
  memcpy(&cur_entry_header, &cursor_file[sizeof(cur_header_type) + (numb_read_curs - 1)*sizeof(cur_entry_header_type)], sizeof(cur_entry_header_type));
  memcpy(&bitmap_info_header, &cursor_file[cur_entry_header.dwImageOffset], sizeof(bitmap_info_header_type));
  
  int size_image, offset_Image;
  // прочитать изображение курсора
  size_image = (cur_entry_header.bHeight*cur_entry_header.bWidth)>>1;			// для 4х-битных курсоров
  offset_Image = cur_entry_header.dwImageOffset + bitmap_info_header.biSize + 16*4;	// 16*4 - смещение на палитру для 16 цветного рисунка
  
  /*char*/unsigned int byte_curs, byte_pict, byte_mask;
  long int int_curs;
  for(int i=0; i<8*8; i++) {
    // Варианты загрузки курсора:
    // 1. формирование цветового растра и наложение маски
    // 2. формирование цветового растра с прозрачностью по нулевому значению цвета
    
    // Формирование двойного слова (4 байта) формата LCD_CURSOR_MEM
    int_curs = 0;
    for (int j=0; j<2; j++) {
      for (int k=0; k<4; k++) {
	byte_pict = cursor_file[offset_Image + i*8 + j*4 + k];
	byte_mask = cursor_file[offset_Image + size_image + i*2 + j];
	
	if (byte_mask & (1 << (7 - k*2)))
	  byte_curs = vpout_cur_color_clear;
	else {
	  /*if ((byte_pict & 0xF0) == 0)
	    byte_curs = vpout_cur_color_clear;
	  else*/ if ((byte_pict & 0xF0) >= 0x80)
	    byte_curs = vpout_cur_color1;
	  else
	    byte_curs = vpout_cur_color0;
	}
	int_curs |= byte_curs << (j*16 + k*4);
	
	if (byte_mask & (1 << (6 - k*2)))
	  byte_curs = vpout_cur_color_clear;
	else {
	  /*if ((byte_pict & 0x0F) == 0)
	    byte_curs = vpout_cur_color_clear;
	  else*/ if ((byte_pict & 0x0F) >= 0x08)
	    byte_curs = vpout_cur_color1;
	  else
	    byte_curs = vpout_cur_color0;
	}
	int_curs |= byte_curs << (j*16 + k*4 + 2);
      }
    }
    this->set_cursor_mem_dword((31 - i/2)*2 + i%2, int_curs);
  }

// Установка точек привязки курсора
  LCDCONTROLLER->LCDXYP = LCD_CONTROLLER_MODEL_LCDXYP_XPOS(cur_entry_header.wPlanes) |
    LCD_CONTROLLER_MODEL_LCDXYP_YPOS(cur_entry_header.wBitCount);
}

void VPOUT_cl::set_dsi_ctr(/*vpout_dsi_ctr*/int dsi_ctr) {
  LCDCONTROLLER->LCD_DSI_CTR = dsi_ctr;
}